import json
from io import StringIO

from flask import request, jsonify, Response, current_app
from flask_restx import Resource, Namespace, reqparse
from sqlalchemy import select, func
from sqlalchemy.orm import Session

from app.defines import StatesCode, Module, OperationType
from app.modle.log import Log
from app.utils.jwt_util import login_required
from app.utils.save_log import save_log
from app.utils.util import to_dict

ns = Namespace('log', description='日志管理接口')

log_list = reqparse.RequestParser(bundle_errors=True)
log_list.add_argument(name='page_size', type=int, location='args', required=False, help='每页记录数量，默认:20')
log_list.add_argument(name='page', type=int, location='args', required=False, help='第几页')
log_list.add_argument(name='module', type=str, location='args', required=False, help='系统模块')
log_list.add_argument(name='name', type=str, location='args', required=False, help='操作人员')
log_list.add_argument(name='operation_type', type=str, location='args', required=False, help='操作类型')
log_list.add_argument(name='operation_status', type=str, location='args', required=False, help='操作状态')
log_list.add_argument(name='start_time', type=str, location='args', required=False, help='开始时间')
log_list.add_argument(name='end_time', type=str, location='args', required=False, help='结束时间')


@ns.route('/log_list')
class LogList(Resource):
    method_decorators = [login_required]

    @ns.doc(id='log', description='日志列表')
    @ns.expect(log_list)
    def get(self):
        """获取日志列表"""
        page_size = int(request.args.get('page_size', 20))
        page = int(request.args.get('page', 1))

        module = request.args.get('module')
        name = request.args.get('name')
        operation_type = request.args.get('operation_type')
        operation_status = request.args.get('operation_status')
        start_time = request.args.get('start_time')
        end_time = request.args.get('end_time')

        with Session(current_app.engine) as session:
            stmt = select(Log)

            count = select(func.count(Log.id))
            count_results = session.execute(count).scalars().first()

            if module:
                stmt = stmt.where(Log.module == module)
            if name:
                stmt = stmt.where(Log.operation_staff == name)
            if operation_type:
                stmt = stmt.where(Log.operation_type == operation_type)
            if operation_status:
                stmt = stmt.where(Log.operation_status == operation_status)
            if start_time and end_time:
                stmt = stmt.where(Log.operation_time.between(start_time, end_time))

            stmt = stmt.offset(page_size * (page - 1)).limit(page_size)
            results = session.execute(stmt).scalars().all()

        save_log(request, Module.LOG, OperationType.INQUIRE, StatesCode.SUCCESS)

        return jsonify(code=StatesCode.SUCCESS, message='成功', total=count_results, data=to_dict(results))


@ns.route('/log_export')
class LogExportApi(Resource):
    method_decorators = [login_required]

    @ns.doc(id='log_export', description='日志导出')
    def get(self):
        """日志导出"""
        with Session(current_app.engine) as session:
            stmt = select(Log)
            results = session.execute(stmt).scalars().all()

        response = Response(StringIO(json.dumps(to_dict(results))))
        response.headers['Content-Type'] = 'application/octet-stream'
        response.headers['Content-Disposition'] = 'attachment; filename=%s' % 'log.json'

        save_log(request, Module.LOG, OperationType.EXPORT, StatesCode.SUCCESS)

        return response
